import pandas as pd
import numpy as np
from pandas.util.testing import assert_frame_equal
from ovretl.loads_utils.calculate_single_load_total_quantities import calculate_single_load_total_quantities


def test_calculate_single_loads_total_quantities():
    loads_df = pd.DataFrame(
        data={
            "unit_weight": [10, 100, 10, 10],
            "unit_length": [120, 120, 120, 120],
            "unit_height": [100, 100, 100, 100],
            "unit_width": [80, 80, 80, 80],
            "unit_number": [1, 1, 1, 1],
            "total_weight": [200, np.nan, 200, 200],
            "total_volume": [0.9, 0.9, 1, np.nan],
        }
    )
    loads_df_with_total_quantities = pd.DataFrame(
        data={
            "unit_weight": [10, 100, 10, 10],
            "unit_length": [120, 120, 120, 120],
            "unit_height": [100, 100, 100, 100],
            "unit_width": [80, 80, 80, 80],
            "unit_number": [1, 1, 1, 1],
            "total_weight": [200, 100, 200, 200],
            "total_volume": [0.9, 0.9, 1, 0.96],
            "total_number": [1, 1, 1, 1],
        }
    )
    loads_df = loads_df.apply(calculate_single_load_total_quantities, axis=1)
    assert_frame_equal(loads_df, loads_df_with_total_quantities, check_dtype=False)



import pandas as pd


def merge_with_propositions_loads_then_with_shipments_loads(
    shipments_df: pd.DataFrame, propositions_loads_df: pd.DataFrame, shipments_loads_df: pd.DataFrame,
) -> pd.DataFrame:
    shipments_df = pd.merge(
        shipments_df,
        propositions_loads_df[
            [
                "total_number",
                "total_volume",
                "total_weight",
                "taxable_weight",
                "weight_measurable",
                "hazardous",
                "non_stackable",
                "proposition_id",
            ]
        ],
        how="left",
        on="proposition_id",
    )

    shipments_df_with_total_weight = shipments_df[~shipments_df["total_weight"].isnull()]
    shipments_df_without_total_weight = shipments_df[shipments_df["total_weight"].isnull()].drop(
        [
            "total_number",
            "total_weight",
            "total_volume",
            "taxable_weight",
            "weight_measurable",
            "hazardous",
            "non_stackable",
        ],
        axis=1,
    )

    shipments_df_without_total_weight = pd.merge(
        shipments_df_without_total_weight,
        shipments_loads_df[
            [
                "total_number",
                "total_volume",
                "total_weight",
                "taxable_weight",
                "weight_measurable",
                "hazardous",
                "non_stackable",
                "shipment_id",
            ]
        ],
        how="left",
        on="shipment_id",
    )

    return (
        pd.concat([shipments_df_with_total_weight, shipments_df_without_total_weight], sort=False)
        .reset_index(drop=True)
        .rename(columns={"hazardous": "loads_hazardous"})
    )


def split_propositions_shipments_loads(loads_df: pd.DataFrame):
    propositions_loads_df = loads_df[~loads_df["proposition_id"].isnull()]
    shipments_loads_df = loads_df[~loads_df["shipment_id"].isnull()]
    return propositions_loads_df, shipments_loads_df


def merge_shipments_with_loads(shipments_df: pd.DataFrame, loads_df: pd.DataFrame):
    (propositions_loads_df, shipments_loads_df,) = split_propositions_shipments_loads(loads_df)
    shipments_df = merge_with_propositions_loads_then_with_shipments_loads(
        shipments_df=shipments_df, propositions_loads_df=propositions_loads_df, shipments_loads_df=shipments_loads_df,
    )
    shipments_df = shipments_df.sort_values(by=["total_weight", "total_volume"], ascending=False).drop_duplicates(
        subset=["shipment_id"]
    )
    return shipments_df



import pandas as pd


def cq_missing_categories(computed_quotations_df: pd.DataFrame):
    category_complete = computed_quotations_df.groupby("category").apply(
        lambda group: group["kronos_selected"].any() or group["selected"].any()
    )
    if False in category_complete.values:
        return category_complete.value_counts()[False]
    return 0


def add_missing_categories(computed_quotations_with_ocp_df: pd.DataFrame):
    cq_fullness = computed_quotations_with_ocp_df.groupby(by=["computed_quotation_id"]).apply(cq_missing_categories)

    computed_quotations_with_ocp_df = pd.merge(
        computed_quotations_with_ocp_df,
        cq_fullness.to_frame(name="missing_categories"),
        left_on="computed_quotation_id",
        right_index=True,
    )
    computed_quotations_with_ocp_df["full"] = computed_quotations_with_ocp_df["missing_categories"].apply(
        lambda x: x == 0
    )
    return computed_quotations_with_ocp_df.astype({"proposition_id": int})



import pandas as pd


def find_shipment_category_shipowner(shipowners_associated_df: pd.DataFrame, role: str):
    if len(shipowners_associated_df) == 0:
        return None
    mask_role = shipowners_associated_df["role"] == role
    shipowners_associated_df_filtered = shipowners_associated_df[mask_role].reset_index(
        drop=True
    )
    if len(shipowners_associated_df_filtered) > 0:
        return shipowners_associated_df_filtered.loc[0, "name"]
    return None


def add_shipowners_to_shipment(
    shipment: pd.Series, shipowner_shipment_with_name_df: pd.DataFrame
) -> pd.Series:
    mask_shipment = (
        shipowner_shipment_with_name_df["shipment_id"] == shipment["shipment_id"]
    )
    shipment["pickup_shipowner"] = find_shipment_category_shipowner(
        shipowners_associated_df=shipowner_shipment_with_name_df[mask_shipment],
        role="departure_truck_freight",
    )
    shipment["departure_shipowner"] = find_shipment_category_shipowner(
        shipowners_associated_df=shipowner_shipment_with_name_df[mask_shipment],
        role="departure_fees",
    )
    shipment["freight_shipowner"] = find_shipment_category_shipowner(
        shipowners_associated_df=shipowner_shipment_with_name_df[mask_shipment],
        role="freight",
    )
    shipment["arrival_shipowner"] = find_shipment_category_shipowner(
        shipowners_associated_df=shipowner_shipment_with_name_df[mask_shipment],
        role="arrival_fees",
    )
    shipment["delivery_shipowner"] = find_shipment_category_shipowner(
        shipowners_associated_df=shipowner_shipment_with_name_df[mask_shipment],
        role="arrival_truck_freight",
    )
    return shipment



import pandas as pd
import numpy as np


from pandas.util.testing import assert_frame_equal
from ovretl.employees_utils.find_shipment_employee_name import add_employees_to_shipments


def test_add_employees_to_shipments():
    shipments_df = pd.DataFrame(data={"sales_owner_id": [1], "operations_owner_id": [np.nan], "pricing_owner_id": [3],})
    employees_df = pd.DataFrame(data={"id": [1, 2, 3], "name": ["Gautier", "Oria", "Jean"]})
    result = add_employees_to_shipments(shipments_df, employees_df)
    result_should_be = pd.DataFrame(data={"sales": ["Gautier"], "operations": [np.nan], "pricing": ["Jean"],})
    assert_frame_equal(result, result_should_be, check_dtype=False)



import pandas as pd
from datetime import datetime


def check_shipment_active(shipment: pd.Series, shipment_events: pd.DataFrame):
    if shipment["cancelled"] == True:
        return False
    if shipment["shipment_status"] in [
        "new",
        "not_answered",
        "propositions_sent",
        "purchase_ready",
        "to_be_requoted",
        "not_answered_declined",
    ]:
        return False
    if shipment["shipment_status"] == "finished" or not pd.isna(shipment["finished_date"]):
        return False
    if len(shipment_events) > 0 and (shipment_events["date_type"] == "actual").all():
        return False
    if (
        pd.isna(shipment["pickup_date"])
        and pd.isna(shipment["departure_date"])
        and pd.isna(shipment["arrival_date"])
        and pd.isna(shipment["delivery_date"])
    ):
        return True
    if not pd.isna(shipment["pickup_date"]) and pd.isna(shipment["departure_date"]):
        return True
    if not pd.isna(shipment["arrival_date"]) and (datetime.now() - pd.to_datetime(shipment["arrival_date"])).days > -10:
        return True
    if (
        not pd.isna(shipment["departure_date"])
        and (datetime.now() - pd.to_datetime(shipment["departure_date"])).days < 2
    ):
        return True
    return False



import pandas as pd
from pandas.util.testing import assert_series_equal
from ovretl import calculate_margins, calculate_shipment_prices

df_prices = pd.DataFrame(
    data={
        "category": [
            "departure_truck_freight",
            "arrival_truck_freight",
            "departure_fees",
            "freight",
            "arrival_fees",
            "other",
            "insurance",
        ],
        "price_in_eur": [1336.05, 680, 108.59, 3651, 95, 1000, 400],
        "margin_price_in_eur": [0, 20, 32, 534, 0, 1000, 400],
        "vat_price_in_eur": [0, 10, 0, 20, 0, 0, 0],
    },
)
df_prices_initial = pd.DataFrame(
    data={
        "category": [
            "departure_truck_freight",
            "arrival_truck_freight",
            "departure_fees",
            "freight",
            "arrival_fees",
            "other",
            "insurance",
        ],
        "price_in_eur": [1336.05, 680, 108.59, 3651, 95, 1000, 500],
        "margin_price_in_eur": [0, 10, 32, 534, 0, 1000, 500],
        "vat_price_in_eur": [0, 10, 0, 20, 0, 0, 0],
    },
)


def test_calculate_margins():
    shipment = pd.Series()

    result = calculate_margins(shipment, df_prices, df_prices_initial)
    result_should_be = pd.Series(
        data=[576, 500, 586, 400],
        index=[
            "initial_margin_without_insurance",
            "initial_margin_insurance",
            "margin_without_insurance",
            "margin_insurance",
        ],
    )
    assert_series_equal(result, result_should_be)


def test_calculate_shipment_prices():
    shipment = pd.Series()

    result = calculate_shipment_prices(shipment, df_prices)
    result_should_be = pd.Series(
        data=[1336.05, 108.59, 3651, 95, 680, 400, 1336.05 + 108.59 + 3651 + 95 + 680 + 400, 0, 1000, 30],
        index=[
            "departure_truck_freight_price",
            "departure_fees_price",
            "freight_price",
            "arrival_fees_price",
            "arrival_truck_freight_price",
            "insurance_price",
            "turnover",
            "customs_price",
            "other_price",
            "vat_price",
        ],
    )
    assert_series_equal(result, result_should_be)



import pandas as pd

from typing import Optional, Tuple


def find_prices_from_entity(
    entity_df: pd.DataFrame, prices_df: pd.DataFrame, entity_key_in_prices_df: str, entity_key_in_entity_df="id",
) -> Optional[pd.DataFrame]:
    if len(entity_df) > 0:
        entity_ids = entity_df.loc[:, entity_key_in_entity_df]
        return prices_df[prices_df[entity_key_in_prices_df].isin(entity_ids)]
    return None


def find_most_relevant_prices_factory(
    prices_propositions_by_category_df: pd.DataFrame,
    final_checks_df: pd.DataFrame,
    prices_final_check_by_category_df: pd.DataFrame,
    billings_df: pd.DataFrame,
    prices_billings_by_category_df: pd.DataFrame,
):
    def find_most_relevant_prices(
        shipment_id: str, proposition_id: str, initial_proposition=False
    ) -> Tuple[Optional[pd.DataFrame], Optional[str]]:
        if initial_proposition == True:
            final_checks_df_filtered = final_checks_df[final_checks_df["shipment_id"] == shipment_id].reset_index(
                drop=True
            )
            prices_initial_proposition_filtered = find_prices_from_entity(
                entity_df=final_checks_df_filtered,
                prices_df=prices_propositions_by_category_df,
                entity_key_in_prices_df="proposition_id",
                entity_key_in_entity_df="initial_proposition_id",
            )
            if prices_initial_proposition_filtered is not None and len(prices_initial_proposition_filtered) > 0:
                return prices_initial_proposition_filtered, "final_check"

        final_checks_df_filtered = final_checks_df[final_checks_df["shipment_id"] == shipment_id].reset_index(drop=True)
        prices_final_check_filtered = find_prices_from_entity(
            entity_df=final_checks_df_filtered,
            prices_df=prices_final_check_by_category_df,
            entity_key_in_prices_df="final_check_id",
        )
        if prices_final_check_filtered is not None and len(prices_final_check_filtered) > 0:
            return prices_final_check_filtered, "final_check"

        billings_df_filtered = billings_df[billings_df["shipment_id"] == shipment_id].reset_index(drop=True)
        prices_billings_filtered = find_prices_from_entity(
            entity_df=billings_df_filtered,
            prices_df=prices_billings_by_category_df,
            entity_key_in_prices_df="billing_id",
        )
        if prices_billings_filtered is not None and len(prices_billings_filtered) > 0:
            return prices_billings_filtered, "billing"

        if not pd.isna(proposition_id):
            return (
                prices_propositions_by_category_df[
                    prices_propositions_by_category_df["proposition_id"] == proposition_id
                ],
                "proposition",
            )
        return None, None

    return find_most_relevant_prices



import pandas as pd
from ovretl.billings_utils.compute_billing_status import compute_billing_status


def test_compute_billing_status():
    billings_df_awaiting_invoice = pd.DataFrame(
        data={
            "shipment_id": ["0", "1", "0"],
            "status": ["new", "DEF", "in_modification"],
        }
    )
    billing_status_awaiting_invoice = compute_billing_status(
        billings_df_awaiting_invoice, "0"
    )
    billings_df_due = pd.DataFrame(
        data={
            "shipment_id": ["0", "1", "0"],
            "status": ["due", "DEF", "in_modification"],
        }
    )
    billing_status_due = compute_billing_status(billings_df_due, "0")
    billings_df_paid = pd.DataFrame(
        data={
            "shipment_id": ["0", "1", "0"],
            "status": ["paid", "DEF", "in_modification"],
        }
    )
    billing_status_paid = compute_billing_status(billings_df_paid, "0")
    assert billing_status_awaiting_invoice == "awaiting_invoice"
    assert billing_status_due == "due"
    assert billing_status_paid == "paid"
